#Jerome Remigio
#Started: Oct 18, 2013
#Missile Command Clone

import pygame, random, os, sys, math
from pygame.locals import *

pygame.init()

# COLOURS ---------------

white = (255, 255, 255)
black = (0, 0, 0)
gold = (255, 215, 0)
green = (188, 238, 104)

#---------------------------------
cross_hair = ("      XXXX      ",
              "    XXXXXXXX    ",
              "  XXX..XX..XXX  ",
              " XX....XX....XX ",
              " X.....XX.....X ",
              "XX.....XX.....XX",
              "X......XX......X",
              "XXXXXXXXXXXXXXXX",
              "XXXXXXXXXXXXXXXX",
              "X......XX......X",
              "XX.....XX.....XX",
              " X.....XX.....X ",
              " XX....XX....XX ",
              "  XXX..XX..XXX  ",
              "    XXXXXXXX    ",
              "      XXXX      ")


# SPRITE GROUPS ---------------------------
cloud_group = pygame.sprite.Group()
lightning_group = pygame.sprite.Group()
button_group = pygame.sprite.Group()

player_group = pygame.sprite.GroupSingle()
building_group = pygame.sprite.Group()
artillery_group  =pygame.sprite.Group()

asteroid_group = pygame.sprite.Group()

explosion_group = pygame.sprite.Group()

#------------------------------------------

def load_image(name, colorkey=None):
    
    try:
        image = pygame.image.load(name)
        
    except pygame.error, message:
        print('Cannot load image:', name)
        raise SystemExit, message
    
    image = image.convert()
    
    if colorkey is not None:
        if colorkey is -1:
            colorkey = image.get_at((0,0))
            
        image.set_colorkey(colorkey, RLEACCEL)
        
    return image, image.get_rect()

#---------------------------------

class Game_Window():

    def __init__(self, size):
        self.size = size
        self.screen = pygame.display.set_mode(size)
        self.score = 0
        self.level = 1
        self.repair = 0
        self.start_screen = 1
 #       self.help_screen = 0
        self.status_screen = 0
        self.game_over = 0
 #       self.game_started = 0
        self.spawn_rate = range(200)
        self.clock = pygame.time.Clock()
        self.start_screen_image, self.rect = load_image('start_screen.jpg', -1)
        self.background, null = load_image('background.jpg', -1)
        self.title, null = load_image('title.png', -1)
        self.max_asteroids = 5
        self.asteroid_count = 0
        self.updated_score = 0
        self.got_high_score = 0
        self.got_points = 0
        self.move_left = 1

    def display(self):

        if self.move_left:
            self.rect.x -= 1

            if self.rect.x == -500:
                self.move_left = 0

        else:
            self.rect.x += 1
            if self.rect.x == 0:
                self.move_left = 1
                
        self.screen.blit(self.start_screen_image, self.rect.topleft)

    def initialize(self):
        self.level += 1
        self.max_asteroids += 2
        self.asteroid_count = 0

    def reset(self):
        self.score = 0
        self.level = 1
        self.repair = 0
        self.max_asteroids = 5
        self.asteroid_count = 0
        player.ammo = 15
        player.super_ammo = 0
        self.game_over = 0
        self.updated_score = 0
        self.got_high_score = 0
        asteroid_group.empty()

        for building in building_group:
            building.health = 2

    def check_building_status(self):
        check = 0
        for building in building_group:
            if building.health == 0:
                check += 1

        if check == 6:
            return True
        else:
            return False

    def spawn_asteroid(self):
        asteroid = Asteroid(random.randint(2, 4))
        self.asteroid_count += 1
        asteroid.rect.centerx = random.randint(-50, 1050)
        asteroid.rect.centery = -35

        destination = random.choice([building.rect.midbottom for building in building_group])

        x_comp = asteroid.rect.centerx - destination[0]
        y_comp = destination[1] - asteroid.rect.centery

        angle = math.atan2(x_comp, y_comp)

        asteroid.image = pygame.transform.rotate(asteroid.image, -math.degrees(angle))

        asteroid.angle = angle + math.pi

    def count_bonus_points(self):
        repair = 0
        score = 0
        super_ammo = 0
        missile_points = player.ammo * 10
        artillery_points = player.super_ammo * 15
        
        for building in building_group:
            if building.health == 2:
                repair += 5
                score += 10
                super_ammo += 1

            elif building.health >= 1:
                repair += 5

        return repair, score, super_ammo, missile_points, artillery_points

    def repair_building(self, pos):

        if self.repair >= 50:
            for building in building_group:
                if building.rect.left < pos[0] < building.rect.right and\
                   building.rect.top < pos[1] < building.rect.bottom:

                    if building.health < 2:
                        building.health += 1
                        self.repair -= 50

    def press_button(self, pos):
        for button in button_group:
            if button.rect.left < pos[0] < button.rect.right and\
               button.rect.top < pos[1] < button.rect.bottom:

                if not button.pressed:
                    button.pressed = 1

    def unpress_button(self, pos):
        for button in button_group:
            if button.rect.left < pos[0] < button.rect.right and\
               button.rect.top < pos[1] < button.rect.bottom:

                if button.pressed:
                    button.command()
                    button.pressed = 0

            else:
                for button in button_group:
                    if button.pressed:
                        button.pressed = 0

    def show_text(self, file_name, font_size, message, aa, colour, (x, y)):
        font = pygame.font.Font(file_name, font_size)
        text = font.render(message, aa, colour)
        self.screen.blit(text, (x, y))


    def show_ammo(self):
        self.show_text(None, 25, 'MISSILES: ' + str(player.ammo), 1, gold, (365, 720))

    def show_super_ammo(self):
        self.show_text(None, 25, 'ARTILLERY: ' + str(player.super_ammo), 1, gold, (515, 720))

    def show_level(self):
        self.show_text(None, 25, 'LEVEL: ' + str(window.level), 1, gold, (250,10))

    def show_score(self):
        self.show_text(None, 25, 'SCORE: ' + str(window.score), 1, gold, (460, 10))

    def show_repair_points(self):
        self.show_text(None, 25, 'REPAIR: ' + str(window.repair), 1, gold, (670, 10))

    def show_status_screen(self, repair, building_pts, super_ammo, missile_pts, artillery_pts):
        pygame.draw.rect(window.screen, green, (250, 50, 500, 450))
        pygame.draw.rect(window.screen, black, (250, 50, 500, 450), 6)
        
        if not self.got_points:
            self.got_points = 1

        self.show_text(None, 20, "BUILDING BONUS:                            +"\
                         + str(building_pts), 1, black, (350, 100))

        self.show_text(None, 20, "MISSILE BONUS:                              +"\
                         + str(missile_pts), 1, black, (350, 140))

        self.show_text(None, 20, "ARTILLERY BONUS:                         +"\
                         + str(artillery_pts), 1, black, (350, 180))

        self.show_text(None, 20, "REPAIR POINTS:                              +"\
                         + str(repair), 1, black, (350, 220))

        self.show_text(None, 20, "ARTILLERY AMMO:                           +"\
                         + str(super_ammo), 1, black, (350, 260))

        self.show_text(None, 25, "TOTAL BONUS:                              +"\
                         + str(building_pts+missile_pts+artillery_pts), 1, black, (350, 300))

        self.show_text(None, 35, "'Click on any building to repair it'", 1, black, (305, 360))

        self.show_text(None, 30, "(50 points per repair)", 1, black, (400, 400))

        self.show_text(None, 40, "Press 'Enter' to Advance", 1, black, (330, 430))

    def game_over_screen(self):
        pygame.draw.rect(window.screen, green, (300, 200, 400, 300))
        pygame.draw.rect(window.screen, black, (300, 200, 400, 300), 6)

        self.show_text(None, 45, "GAME OVER", 1, black, (400, 240))

        self.show_text(None, 30, "ALL THE BUILDINGS ARE DESTROYED", 1, black, (312, 290))

        self.show_text(None, 35, "Press 'ENTER' to restart", 1, black, (370, 450))

    def check_scores(self, score):
        scores_file = open('high_scores.txt', 'r')
        data = []
        entries = []

        for line in scores_file:
            data.append(line)
            entries.append(line.split())

        scores_file.close()

        if len(data) < 10:
            return True, data, entries

        else:
            for entry in entries:
                if score > int(entry[1]):
                    return True, data, entries
            return False, [], []


    def update_high_scores(self, score, data, entries):
        name = raw_input('Enter your name (DO NOT include spaces): ')

        updated = 0

        scores_file = open('high_scores.txt', 'w')

        if entries == []:
            data.append(name + '\t' + str(score) + '\n')
            scores_file.writelines(data)

        else:
            for i in range(len(entries)):
                if score > int(entries[i][1]):
                    if len(entries) == 10:
                        data.insert(i, name + '\t' + str(score) + '\n')
                        del data[-1]
                        scores_file.writelines(data)
                        updated = 1
                        break

                    else:
                        data.insert(i, name + '\t' + str(score) + '\n')
                        scores_file.writelines(data)
                        updated = 1
                        break

            if not updated:
                if not len(entries) == 10:
                    data.append(name + '\t' + str(score) + '\n')
                    scores_file.writelines(data)

                else:
                    scores_file.writelines(data)

        scores_file.close()
        self.updated_score = 1
        

# GAME LOOP START SCREEN ---------------------------------------------------------------
    def start_screen_loop(self):
        x = 60
        for i in range(3):
            cloud = Cloud()
            cloud.rect.top = random.randint(10, 25)
            cloud.rect.left = x
            x += 300

        start_button = Start_Button()
        start_button.rect.topleft = (225, 550)
        
##        help_button = Help_Button()
##        help_button.rect.topleft = (456, 550)
##        
##        score_button = Score_Button()
##        score_button.rect.topleft = (687, 550)

        while self.start_screen:
        #----------- EVENT HANDLERS --------------------------------

            for event in pygame.event.get():
                
                if event.type == QUIT:
                    pygame.quit()
                    sys.exit()

                elif event.type == MOUSEBUTTONDOWN:
                    pos = pygame.mouse.get_pos()
                    self.press_button(pos)

                elif event.type == MOUSEBUTTONUP:
                    pos = pygame.mouse.get_pos()
                    self.unpress_button(pos)

        #----------- GAME LOGIC ------------------------------------

            

        #----------- DRAW TO SCREEN ----------------------------------------------------
                
            window.display()

            for lightning in lightning_group:
                lightning.display()

            for cloud in cloud_group:
                cloud.spawn_lightning()
                cloud.display()

            for button in button_group:
                button.display()

            self.screen.blit(self.title, (100, 270))


            self.clock.tick(60)

            pygame.display.flip()


#---------------------------------------
class Cloud(pygame.sprite.Sprite):

    def __init__(self):
        pygame.sprite.Sprite.__init__(self, cloud_group)
        self.ltn_spawn_rate = range(150)
        self.image, self.rect = load_image('cloud.png', -1)

    def spawn_lightning(self):
        if random.choice(self.ltn_spawn_rate) == 1:
            lightning = Lightning()

            lightning.rect.midtop = self.rect.center

    def display(self):
        window.screen.blit(self.image, self.rect.topleft)
        
        
            
class Lightning(pygame.sprite.Sprite):

    def __init__(self):
        pygame.sprite.Sprite.__init__(self, lightning_group)
        self.duration_time = 103
        self.lightning1, self.rect = load_image('lightning1.png', -1)
        self.lightning2, null = load_image('lightning2.png', -1)
        self.lightning3, null = load_image('lightning3.png', -1)
        self.lightning4, null = load_image('lightning4.png', -1)
        self.lightning5, null = load_image('lightning5.png', -1)
        self.lightning6, null = load_image('lightning6.png', -1)

    def draw(self, image):
        window.screen.blit(image, self.rect.topleft)

    def display(self):
        self.duration_time -= 8
        
        if self.duration_time > 85:
            self.draw(self.lightning1)

        elif self.duration_time > 68:
            self.draw(self.lightning2)

        elif self.duration_time > 51:
            self.draw(self.lightning3)

        elif self.duration_time > 34:
            self.draw(self.lightning4)

        elif self.duration_time > 17:
            self.draw(self.lightning5)

        elif self.duration_time > 0:
            self.draw(self.lightning6)

        else:
            self.remove(lightning_group)


class Button():#pygame.sprite.Sprite):

    def __init__(self, image_name1, image_name2):
        pygame.sprite.Sprite.__init__(self, button_group)
        self.image1, self.rect = load_image(image_name1, -1)
        self.image2, null = load_image(image_name2, -1)
        self.pressed = 0

    def display(self):
        if not self.pressed:
            window.screen.blit(self.image1, self.rect.topleft)

        else:
            window.screen.blit(self.image2, self.rect.topleft)
            
class Start_Button(Button, pygame.sprite.Sprite):

    def __init__(self):
        Button.__init__(self, 'start_button.png', 'pressed_start_button.png')

    def command(self):
        window.start_screen = 0
        cloud_group.empty()
        button_group.empty()

##class Help_Button(Button, pygame.sprite.Sprite):
##    
##    def __init__(self):
##        Button.__init__(self, 'help_button.png', 'pressed_help_button.png')
##
##    def command(self):
##        window.help_screen = 1
##
##class Score_Button(Button, pygame.sprite.Sprite):
##
##    def __init__(self):
##        Button.__init__(self, 'high_score_button.png', 'pressed_high_score_button.png')
##
##    def command(self):
##        window.show_high_scores()  #create function

##class Pause_Button(Button, pygame.sprite.Sprite):
##
##    def __init__(self):
##        Button.__init__(self, 'pause_button.png', 'pressed_pause_button.png')
##
##    def command(self):
##        window.pause_game()     #create function
##
##class Title_Button(Button, pygame.sprite.Sprite):
##
##    def __init__(self):
##        Button.__init__(self, 'title_button.png', 'pressed_title_button.png')
##
##    def command(self):
##        window.reset()
##        window.start_screen = 1
##        window.game_started = 0
##        button_group.empty()
        
        

#-------------------------------------------
        
class Player(pygame.sprite.Sprite):

    def __init__(self):
        pygame.sprite.Sprite.__init__(self, player_group)
        self.ammo = 15
        self.super_ammo = 0
        self.super_artillery = False
        self.image, self.rect = load_image('player.png', -1)

    def shoot(self, x, y, radius, speed, mouse_pos):
        y_comp = y - mouse_pos[1]
        x_comp = mouse_pos[0] - x

        angle = math.atan2(x_comp, y_comp)

        artillery = Artillery(x, y, radius, angle, speed, mouse_pos)
        


class Artillery(pygame.sprite.Sprite):

    def __init__(self, x, y, radius, angle, speed, destination):
        pygame.sprite.Sprite.__init__(self, artillery_group)
        self.radius = radius
        self.angle = angle
        self.speed = speed
        self.destination = destination
        self.rect = pygame.Rect(x-radius, y-radius, radius*2, radius*2)

    def update(self):
        self.move()
        self.collide()
        self.check_boundaries()
        self.display()

    def move(self):
        self.rect.x += self.speed * math.sin(self.angle)
        self.rect.y -= self.speed * math.cos(self.angle)

    def collide(self):
        asteroid_list = pygame.sprite.spritecollide(self, asteroid_group, 1)

        if asteroid_list != []:
            self.explode()
            for asteroid in asteroid_list:
                window.score += 10
                asteroid.explode()
        

    def check_boundaries(self):

        if (self.rect.x < -7 or self.rect.x > window.size[0]) or\
        (self.rect.y > window.size[1] or self.rect.y < -7):
            self.remove(artillery_group)

    def display(self):
        pygame.draw.circle(window.screen, gold, self.rect.center, self.radius)

    def explode(self):
        explosion = Explosion()
        explosion.rect.center = self.rect.center
        self.remove(artillery_group)
        

class Asteroid(pygame.sprite.Sprite):
    
    def __init__(self, speed):
        pygame.sprite.Sprite.__init__(self, asteroid_group)
        self.angle = 0
        self.speed = speed
        self.image, self.rect = load_image('asteroid.png', -1)

    def update(self):
        self.move()
        self.collide()
        self.check_boundaries()
        self.display()

    def move(self):
        self.rect.centerx += self.speed * math.sin(self.angle)
        self.rect.centery -= self.speed * math.cos(self.angle)

    def collide(self):

        building_dict = pygame.sprite.groupcollide(asteroid_group, building_group, 1, 0)

        if building_dict != {}:
            self.explode()
            for x in building_dict.keys():
                for building in building_dict[x]:

                    if building.health > 0:
                        building.health -= 1

                    elif building.health == 0 and window. score >= 5:
                        window.score -= 5

    def check_boundaries(self):
        if self.rect.right < -5 or self.rect.left > window.size[0] + 5:
            self.remove(asteroid_group)

        elif self.rect.bottom > 663:
            self.explode()

            if window.score >= 5:
                window.score -= 5

    def display(self):
        window.screen.blit(self.image, self.rect.center)

    def explode(self):
        explosion = Explosion()
        explosion.rect.center = self.rect.center
        self.remove(asteroid_group)


class Building(pygame.sprite.Sprite):

    def __init__(self):
        pygame.sprite.Sprite.__init__(self, building_group)
        self.health = 2
        self.image1, self.rect = load_image('buildings.png', -1)
        self.image2, null = load_image('damaged_building.png', -1)
        self.image3, null = load_image('destroyed_building.png', -1)

    def display(self):
        if self.health == 2:
            window.screen.blit(self.image1, self.rect.topleft)

        elif self.health == 1:
            window.screen.blit(self.image2, self.rect.topleft)

        else:
            window.screen.blit(self.image3, self.rect.topleft)


class Explosion(pygame.sprite.Sprite):

    def __init__(self):
        pygame.sprite.Sprite.__init__(self, explosion_group)
        self.duration = 200
        self.image1, self.rect = load_image('explosion1.png', -1)
        self.image2, null = load_image('explosion2.png', -1)
        self.image3, null = load_image('explosion3.png', -1)
        self.image4, null = load_image('explosion4.png', -1)
        self.image5, null = load_image('explosion5.png', -1)
        self.image6, null = load_image('explosion6.png', -1)
        self.image7, null = load_image('explosion7.png', -1)
        self.image8, null = load_image('explosion8.png', -1)
        self.image9, null = load_image('explosion9.png', -1)
        self.image10, null = load_image('explosion10.png', -1)
        self.image11, null = load_image('explosion11.png', -1)
        self.image12, null = load_image('explosion12.png', -1)
        self.image13, null = load_image('explosion13.png', -1)
        self.image14, null = load_image('explosion14.png', -1)
        self.image15, null = load_image('explosion15.png', -1)
        self.image16, null = load_image('explosion16.png', -1)
        self.image17, null = load_image('explosion17.png', -1)
        self.image18, null = load_image('explosion18.png', -1)

    def update(self):
        self.collide()
        self.display()

    def collide(self):
        hit_list = pygame.sprite.spritecollide(self, asteroid_group, 1)

        for hit in hit_list:
            asteroid.explode()
            window.score += 10

    def draw(self, image):
        window.screen.blit(image, self.rect.center)

    def display(self):
        self.duration -= 4

        if self.duration > 187:
            self.draw(self.image1)

        elif self.duration > 176:
            self.draw(self.image2)

        elif self.duration > 165:
            self.draw(self.image3)
            
        elif self.duration > 154:
            self.draw(self.image4)
            
        elif self.duration > 143:
            self.draw(self.image5)
            
        elif self.duration > 132:
            self.draw(self.image6)
            
        elif self.duration > 121:
            self.draw(self.image7)
            
        elif self.duration > 110:
            self.draw(self.image8)
            
        elif self.duration > 99:
            self.draw(self.image9)
            
        elif self.duration > 88:
            self.draw(self.image10)
            
        elif self.duration > 77:
            self.draw(self.image11)
            
        elif self.duration > 66:
            self.draw(self.image12)
            
        elif self.duration > 55:
            self.draw(self.image13)
            
        elif self.duration > 44:
            self.draw(self.image14)
            
        elif self.duration > 33:
            self.draw(self.image15)
            
        elif self.duration > 22:
            self.draw(self.image16)
            
        elif self.duration > 11:
            self.draw(self.image17)
            
        elif self.duration > 0:
            self.draw(self.image18)
            
        elif self.duration < 0:
            self.remove(explosion_group)
        
        

#-----------------------------------------------------------------------
            
cursor = pygame.cursors.compile(cross_hair, black = 'X', white = '.')
pygame.mouse.set_cursor((16, 16), (8, 8), *cursor)

pygame.display.set_caption('ARMAGEDDON')


# INSTANCES ------------------------------------

window = Game_Window((1000, 750))

player = Player()
player.rect.center = (window.size[0]/2, 620)


x = 10
y = 580
for i in range(3):
    building = Building()
    building.rect.x = x
    building.rect.y = y
    x += 150

x = 600
for i in range(3):
    building = Building()
    building.rect.x = x
    building.rect.y = y
    x += 150



# MAIN GAME LOOP -------------------------------
while True:

    # EVENT HANDLERS --------------------------------------------

    for event in pygame.event.get():

        if event.type == QUIT:
            pygame.quit()
            sys.exit()

        elif window.status_screen and event.type == KEYDOWN and event.key == K_RETURN:
            
            window.score += building_pts + missile_pts + artillery_pts
            window.repair += repair
            player.super_ammo += super_ammo
            player.ammo += window.max_asteroids
            window.status_screen = 0
            window.got_points = 0
            window.initialize()

        elif window.game_over and event.type == KEYDOWN and event.key == K_RETURN:
            window.reset()

        elif event.type == KEYDOWN and event.key == K_SPACE:
            player.super_artillery = True

        elif event.type == KEYUP and event.key == K_SPACE:
            player.super_artillery = False

        elif not window.status_screen and event.type == MOUSEBUTTONDOWN and player.super_artillery:
            if player.super_ammo > 0:
                player.super_ammo -= 1
                pos = pygame.mouse.get_pos()
                player.shoot(player.rect.midtop[0], player.rect.midtop[1], 7, 14, pos)

        elif not window.status_screen and event.type == MOUSEBUTTONDOWN:
            if player.ammo > 0:
                player.ammo -= 1
                pos = pygame.mouse.get_pos()
                player.shoot(player.rect.midtop[0], player.rect.midtop[1], 7, 8, pos)

        elif window.status_screen and event.type == MOUSEBUTTONDOWN:
            window.repair_building(pygame.mouse.get_pos())


    # GAME LOGIC ------------------------------------------------

    if window.start_screen == 1:
        window.start_screen_loop()

    else:
       # if not window.game_started:
#            window.game_started = 1
   #         title_button = Title_Button()
#            title_button.rect.topleft = (10, 700)
       
        if not (window.status_screen or window.game_over) and (random.choice(window.spawn_rate) < 1\
           and window.asteroid_count < window.max_asteroids):
            window.spawn_asteroid()

        elif window.check_building_status():
            window.game_over = 1
            if not window.updated_score:
                update, data, entries = window.check_scores(window.score)
                if window.got_high_score:
                    window.update_high_scores(window.score, data, entries)
                elif update:
                    window.got_high_score = 1
                    
        elif window.asteroid_count == window.max_asteroids\
             and len(asteroid_group) == 0:
            window.status_screen = 1


    # DRAWING TO THE SCREEN --------------------------------------

    window.screen.blit(window.background, (0, 0))

    pygame.draw.rect(window.screen, black, [0, 663, 1000, 86])

    window.show_text(None, 35, "Hold 'Space Bar' and click to use Artillery", 1, white, (510, 680))
    
    window.screen.blit(player.image, player.rect.topleft)

    window.show_ammo()

    window.show_super_ammo()



    for building in building_group:
        building.display()

    for artillery in artillery_group:
        artillery.update()

    for asteroid in asteroid_group:
        asteroid.update()

    for explosion in explosion_group:
        explosion.update()


    window.show_level()

    window.show_score()

    window.show_repair_points()

    if window.status_screen:
        if not window.got_points:
            repair, building_pts, super_ammo, missile_pts, artillery_pts = window.count_bonus_points()
        window.show_status_screen(repair, building_pts, super_ammo, missile_pts, artillery_pts)

    elif window.game_over:
        window.game_over_screen()
        if window.got_high_score:
            window.show_text(None, 35, "    HIGH SCORE", 1, black, (400, 350))
            window.show_text(None, 35, "Enter name in the python shell", 1, black, (330, 380))
            

    window.clock.tick(60)

    pygame.display.flip()
